// UI Extensibility Application for opening DocuSign sender view from M-Files.
// Depending on selected object and its metadata button to task pane and to right-click menu appears.
// This is configured to work with Desktop and Classic Web clients.

"use strict";

// Selected object variable.
var selected = null;

// Configuration variable.
var buttonConfig = null;

// Workflow states variable.
var workflowConfig = null;

// Objects variable.
var objectsConfig = null;

function OnNewShellUI( shellUI ) {
	// Register to be notified when a new normal shell frame (Event_NewNormalShellFrame) is created.
	shellUI.Events.Register(
		Event_NewNormalShellFrame,
		handleNewNormalShellFrame );
}

function handleNewNormalShellFrame( shellFrame ) {

	// Register to be notified when the shell frame is started.
	// Pass a reference to the function to call when the event is fired.
	shellFrame.Events.Register(
		Event_Started,
		getShellFrameStartedHandler( shellFrame ) );
}

/**
 * Create buttons for Web and Desktop clients.
 *
 * @param {MFiles.ShellFrame} shellFrame - The current shell frame object.
 */
function CreateButtons( shellFrame ) {

    // Create the task pane command.
    var commandSend = shellFrame.Commands.CreateCustomCommand( "Send with DocuSign" );

    // Create the right-click menu command.
    var commandSendMenu = shellFrame.Commands.CreateCustomCommand( "Send with DocuSign" );

    // Set buttons only if buttons configured in settings.
    if( buttonConfig[0] ) {

        // Create the group to the task pane.
        var groupID = shellFrame.TaskPane.CreateGroup( "Sign", 20 );

        // Add command to right group in task pane.
        shellFrame.TaskPane.AddCustomCommandToGroup( commandSend, groupID, 1 );

        // Add to right-click menu.
        shellFrame.Commands.AddCustomCommandToMenu( commandSendMenu, MenuLocation_ContextMenu_Misc2_Top, 1 );
    }

    // Hide all buttons at start, showing is checked when selection changes.
    shellFrame.Commands.SetCommandState(
        commandSend,
        CommandLocation_All,
        CommandState_Hidden
    );

    // Register to be notified when a custom command is clicked.
    shellFrame.Commands.Events.Register(
        Event_CustomCommand,
        function( commandId ) {

            // Check that right button clicked.
            if( commandId === commandSendMenu || commandId === commandSend ) {

                // Open sender view clicked.
                // Address returned by vault application.
                var address = null;

                // Object info to identify object to vault application.
                var item = JSON.stringify( [selected.ObjectVersions[0].ObjVer.Type, selected.ObjectVersions[0].ObjVer.ID, selected.ObjectVersions[0].ObjVer.Version] );

                // Divide handling based on client type.
                if( MFExtApplicationPlatformWeb === MFiles.CurrentApplicationPlatform ) {

                    // Clicked from Classic Web client.
                    // Call Vault extension method.
                    shellFrame.ShellUI.Vault.Async.ExtensionMethodOperations
                        .ExecuteVaultExtensionMethod( "OpenSenderView",
                            item,
                            function( response ) {

                                // Response returns address for browser.
                                address = response;

                                // If address contains "response_type=code" it is consent request page.
                                if( address.indexOf( "response_type=code" ) !== -1 ) {

                                    // Show dialog to inform user that consent request page is opening instead of sender view.
                                    shellFrame.ShowMessage( {
                                        caption: "DocuSign Integration",
                                        message: "DocuSign Integration doesn't have consent to send envelopes on behalf you. Press \"OK\" and give your" +
                                            " DocuSign credentials and allow access. After giving consent, press again the \"Send with DocuSign\"-button.",
                                        button1_title: "OK",
                                        defaultButton: 1
                                    }, function () {

                                        // Open the consent request page in browser if not empty/null.
                                        if( address !== null && address !== "null" ) {
                                            window.open( address );
                                        }
                                    } );
                                }
                                else {

                                    // Normal envelope address so open in browser if not empty/null.
                                    if( address !== null && address !== "null" ) {
                                        window.open( address );
                                    }
                                }
                            },
                            GetExceptionHandler() );
                } else {

                    // Clicked from desktop client.
                    // Call Vault extension method.
                    address = shellFrame.ShellUI.Vault.ExtensionMethodOperations
                        .ExecuteVaultExtensionMethod( "OpenSenderView",
                            item );

                    // If address contains "response_type=code" it is consent request page.
                    if( address.indexOf( "response_type=code" ) !== -1 ) {

                        // Show dialog to inform user that consent request page is opening instead of sender view.
                        shellFrame.ShowMessage( {
                            caption: "DocuSign Integration",
                            message: "DocuSign Integration doesn't have consent to send envelopes on behalf you. Press \"OK\" and give your" +
                                " DocuSign credentials and allow access. After giving consent, press again the \"Send with DocuSign\"-button.",
                            button1_title: "OK",
                            defaultButton: 1
                        } );

                        // Open the consent request page in browser if not empty/null.
                        if( address !== null && address !== "" ) {
                            var shell = new ActiveXObject( "WScript.Shell" );
                            shell.Run( JSON.stringify( address ) );
                        }
                    }
                    else {

                        // Normal envelope address so open in browser if not empty/null.
                        if( address !== null && address !== "" ) {
                            var shell = new ActiveXObject( "WScript.Shell" );
                            shell.Run( JSON.stringify( address ) );
                        }
                    }
                }
            }
        } );

	// Register to be notified when new shell listings are created.
	shellFrame.Events.Register(
		Event_NewShellListing,
		getNewShellListingHandler( shellFrame, commandSend, commandSendMenu ) );

		// Is there already a listing?  If so then we need to hook into it as well.
	if( null !== shellFrame.Listing ) {
		getNewShellListingHandler( shellFrame, commandSend, commandSendMenu )( shellFrame.Listing );
	}
}



// Returns a function that handles the OnStarted event for an IShellFrame.
function getShellFrameStartedHandler( shellFrame ) {

	return function () {
		// The shell frame is now started and can be used.

		if( MFExtApplicationPlatformWeb === MFiles.CurrentApplicationPlatform ) {

			// Clicked from Classic Web client.
			// Get button configuration from vault.
			shellFrame.ShellUI.Vault.Async.ExtensionMethodOperations.ExecuteVaultExtensionMethod( "GetButtonConfiguration",
				"",
				function( response ) {

					// Get button configuration from vault and set buttons based on that.
					buttonConfig = JSON.parse( response );
					CreateButtons( shellFrame );
				},
				GetExceptionHandler() );

			// Get workflow configuration from vault.
			shellFrame.ShellUI.Vault.Async.ExtensionMethodOperations.ExecuteVaultExtensionMethod( "GetWorkflowStatesConfiguration",
				"",
				function( response ) {

					// Get workflow configuration from vault and set buttons based on that.
					workflowConfig = JSON.parse( response );
				},
				GetExceptionHandler() );

			// Get objects configuration from vault and set buttons based on that.
			shellFrame.ShellUI.Vault.Async.ExtensionMethodOperations.ExecuteVaultExtensionMethod( "GetObjectTypesConfiguration",
				"",
				function( response ) {

					// Get object type configuration from vault and set buttons based on that.
					objectsConfig = JSON.parse( response );
				},
				GetExceptionHandler() );

		} else {

			// Clicked from desktop client.
			// Get button configuration from vault and create buttons if wanted.
			buttonConfig = JSON.parse( shellFrame.ShellUI.Vault.ExtensionMethodOperations.ExecuteVaultExtensionMethod( "GetButtonConfiguration", "" ) );
			workflowConfig = JSON.parse( shellFrame.ShellUI.Vault.ExtensionMethodOperations.ExecuteVaultExtensionMethod( "GetWorkflowStatesConfiguration", "" ) );
			objectsConfig = JSON.parse( shellFrame.ShellUI.Vault.ExtensionMethodOperations.ExecuteVaultExtensionMethod( "GetObjectTypesConfiguration", "") );
			CreateButtons( shellFrame );
		}
	}
}

/**
 * Gets a function to handle the NewShellListing event for shell frame. Function handles button hiding.
 *
 * @param {MFiles.ShellFrame} shellFrame - The current shell frame object.
 * @param {Long} commandId - The command id for the first command.
 * @param {Long} command2Id - The command id for the second command.
 *
 * @returns {MFiles.Events.OnNewShellListing} The event handler.
 */
function getNewShellListingHandler(
	shellFrame,
	commandId,
	command2Id ) {

	// Handle button hiding.
	// Return the handler function for NewShellListing event.
	return function( shellListing ) {

		// Listen for selection change events on the listing.
		shellListing.Events.Register(
			Event_SelectionChanged,
			function( selectedItems ) {

				// Sanity.
				if( false === shellListing.IsActive ) {
					return false;
				}

				// Set selected object based on selection.
				selected = selectedItems;

				// Was there only one item selected (and is it an object version)?
				var isOneObjectSelected = selectedItems.Count === 1 && selectedItems.ObjectVersions.Count === 1;

				if( isOneObjectSelected ) {

					// Divide based on client type.
					if( MFExtApplicationPlatformWeb === MFiles.CurrentApplicationPlatform ) {

						// Classic web.
						// Check object permission to possible button hide.
						shellFrame.ShellUI.Vault.Async.ObjectOperations.GetObjectPermissions( selectedItems.ObjectVersions[0].ObjVer,
							function( permissions ) {

								// Get object access control list.
								var acl = permissions.AccessControlList;

								// Has current user edit access to current object.
								shellFrame.ShellUI.Vault.Async.SessionInfo.CheckObjectAccess( acl, MFObjectAccessEdit,
									function( access ) {

										// Get selected object workflow.
										shellFrame.ShellUI.Vault.Async.ObjectPropertyOperations.GetWorkflowState( selectedItems.ObjectVersions[0].ObjVer, true,
											function( workflow ) {

												// Workflow state id.
												var workflowState = workflow.State.Value.GetLookupID();

												// Get type value as id number.
												var type = selectedItems.ObjectVersions[0].ObjVer.Type;

												// Access check done if needed, otherwise allowed permission always.
												var permission = null;
												if( buttonConfig[1] ) {
													permission = access;
												} else {
													permission = true;
												}

												// Show the task menu item only if all conditions are fulfilled.
												shellFrame.Commands.SetCommandState(
													commandId,
													CommandLocation_All,
													( permission &&
														( workflowConfig.length === 0 || workflowState === null || containsObject( workflowState, workflowConfig ) ) &&
														( objectsConfig.length === 0 || containsObject( type, objectsConfig ) ) ) ? CommandState_Active : CommandState_Hidden
												);

												// Show the context menu item only if all conditions are fulfilled.
												shellFrame.Commands.SetCommandState(
													command2Id,
													CommandLocation_All,
													( permission &&
														( workflowConfig.length === 0 || workflowState === null || containsObject( workflowState, workflowConfig ) ) &&
														( objectsConfig.length === 0 || containsObject( type, objectsConfig ) ) ) ? CommandState_Active : CommandState_Hidden
												);

											} );

									} );

							} );
					} else {
						// Desktop client.
						// Check object permission to possible button hide.
						var permissions = shellFrame.ShellUI.Vault.ObjectOperations.GetObjectPermissions( selectedItems.ObjectVersions[0].ObjVer );

						// Get object access control list.
						var acl = permissions.AccessControlList;

						// Has current user edit access to current object.
						var access = shellFrame.ShellUI.Vault.SessionInfo.CheckObjectAccess( acl, MFObjectAccessEdit );

						// Get selected object workflow.
						var workflow = shellFrame.ShellUI.Vault.ObjectPropertyOperations.GetWorkflowState( selectedItems.ObjectVersions[0].ObjVer, true );

						// Workflow state id.
						var workflowState = workflow.State.Value.GetLookupID();

						// Get selected object type.
						var type = selectedItems.ObjectVersions[0].ObjVer.Type;

						// Access check done if needed, otherwise allowed permission always.
						var permission = null;
						if( buttonConfig[1] ) {
							permission = access;
						} else {
							permission = true;
						}

						// Show the task menu item only if all conditions are fulfilled.
						shellFrame.Commands.SetCommandState(
							commandId,
							CommandLocation_All,
							( permission &&
								( workflowConfig.length === 0 || workflowState === 0 || containsObject( workflowState, workflowConfig ) ) &&
								( objectsConfig.length === 0 || containsObject( type, objectsConfig ) ) ) ? CommandState_Active : CommandState_Hidden
						);

						// Show the context menu item only if all conditions are fulfilled.
						shellFrame.Commands.SetCommandState(
							command2Id,
							CommandLocation_All,
							( permission &&
								( workflowConfig.length === 0 || workflowState === 0 || containsObject( workflowState, workflowConfig ) ) &&
								( objectsConfig.length === 0 || containsObject( type, objectsConfig ) ) ) ? CommandState_Active : CommandState_Hidden
						);
					}
				}
				else {

					// Always hide both buttons when multiple selection.
					shellFrame.Commands.SetCommandState(
						commandId,
						CommandLocation_All,
						CommandState_Hidden
					);

					shellFrame.Commands.SetCommandState(
						command2Id,
						CommandLocation_All,
						CommandState_Hidden
					);
				}

				// We succeeded, return true.
				return true;
			} );

		// Register to ShowContextMenu to show button in context menu if allowed. This is done separately to show button also when object selected by right-click.
		shellListing.Events.Register(
			Event_ShowContextMenu,
			function( selectedItems ) {

				// Was there only one item selected (and is it an object version)?
				var isOneObjectSelected = selectedItems.Count === 1 && selectedItems.ObjectVersions.Count === 1;

				if( isOneObjectSelected ) {

					// Divide based on client type.
					if( MFExtApplicationPlatformWeb === MFiles.CurrentApplicationPlatform ) {

						// Classic web.
						// Check object permission to possible button hide.
						shellFrame.ShellUI.Vault.Async.ObjectOperations.GetObjectPermissions( selectedItems.ObjectVersions[0].ObjVer,
							function( permissions ) {

								// Get object access control list.
								var acl = permissions.AccessControlList;

								// Has current user edit access to current object.
								shellFrame.ShellUI.Vault.Async.SessionInfo.CheckObjectAccess( acl, MFObjectAccessEdit,
									function( access ) {

										// Get selected object workflow.
										shellFrame.ShellUI.Vault.Async.ObjectPropertyOperations.GetWorkflowState( selectedItems.ObjectVersions[0].ObjVer, true,
											function( workflow ) {

												// Workflow state id.
												var workflowState = workflow.State.Value.GetLookupID();

												// Get selected object type.
												var type = selectedItems.ObjectVersions[0].ObjVer.Type;

												// Access check done if needed.
												var permission = null;
												if( buttonConfig[1] ) {
													permission = access;
												} else {
													permission = true;
												}

												// Show the task menu item only if all conditions are fulfilled.
												shellFrame.Commands.SetCommandState(
													commandId,
													CommandLocation_All,
													( permission &&
														( workflowConfig.length === 0 || workflowState === null || containsObject( workflowState, workflowConfig ) ) &&
														( objectsConfig.length === 0 || containsObject( type, objectsConfig ) ) ) ? CommandState_Active : CommandState_Hidden
												);

												// Show the context menu item only if all conditions are fulfilled.
												shellFrame.Commands.SetCommandState(
													command2Id,
													CommandLocation_All,
													( permission &&
														( workflowConfig.length === 0 || workflowState === null || containsObject( workflowState, workflowConfig ) ) &&
														( objectsConfig.length === 0 || containsObject( type, objectsConfig ) ) ) ? CommandState_Active : CommandState_Hidden
												);

											} );

									} );

							} );
					} else {

						// Desktop client.
						// Check object permission to possible button hide.
						var permissions = shellFrame.ShellUI.Vault.ObjectOperations.GetObjectPermissions( selectedItems.ObjectVersions[0].ObjVer );

						// Get object access control list.
						var acl = permissions.AccessControlList;

						// Has current user edit access to current object.
						var access = shellFrame.ShellUI.Vault.SessionInfo.CheckObjectAccess( acl, MFObjectAccessEdit );

						// Get selected object workflow.
						var workflow = shellFrame.ShellUI.Vault.ObjectPropertyOperations.GetWorkflowState( selectedItems.ObjectVersions[0].ObjVer, true );

						// Workflow state id.
						var workflowState = workflow.State.Value.GetLookupID();

						// Get selected object type.
						var type = selectedItems.ObjectVersions[0].ObjVer.Type;

						// Access check done if needed.
						var permission = null;
						if( buttonConfig[1] ) {
							permission = access;
						} else {
							permission = true;
						}

						// Show the task menu item only if all conditions are fulfilled.
						shellFrame.Commands.SetCommandState(
							commandId,
							CommandLocation_All,
							( permission &&
								( workflowConfig.length === 0 || workflowState === 0 || containsObject( workflowState, workflowConfig ) ) &&
								( objectsConfig.length === 0 || containsObject( type, objectsConfig ) ) ) ? CommandState_Active : CommandState_Hidden
						);

						// Show the context menu item only if all conditions are fulfilled.
						shellFrame.Commands.SetCommandState(
							command2Id,
							CommandLocation_All,
							( permission &&
								( workflowConfig.length === 0 || workflowState === 0 || containsObject( workflowState, workflowConfig ) ) &&
								( objectsConfig.length === 0 || containsObject( type, objectsConfig ) ) ) ? CommandState_Active : CommandState_Hidden
						);
					}
				}
				else {

					// Always hide both buttons when multiple selection.
					shellFrame.Commands.SetCommandState(
						commandId,
						CommandLocation_All,
						CommandState_Hidden
					);

					shellFrame.Commands.SetCommandState(
						command2Id,
						CommandLocation_All,
						CommandState_Hidden
					);
				}
			} );
	};
}

/// <summary>Function to check whether list contains object or not.</summary>
/// <returns>Boolean value whether object exists in list or not.</returns>
function containsObject( obj, list ) {
	var i;
	for( i = 0; i < list.length; i++ ) {
		if( list[i] === obj ) {
			return true;
		}
	}
	return false;
}

/// <summary>function to return a generic exception handler for exceptions occurring in async calls</summary>
/// <returns>Returns a generic exception handler for exceptions occurring in async calls</returns>
function GetExceptionHandler() {
	return function( shorterror, longerror, errorObject ) {
		MFiles.ReportException( errorObject );
	};
}

